"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamodbFetcher = void 0;
const https_1 = __importDefault(require("https"));
const async_1 = __importDefault(require("async"));
const util_1 = require("util");
const client_dynamodb_1 = require("@aws-sdk/client-dynamodb");
const lib_dynamodb_1 = require("@aws-sdk/lib-dynamodb");
const node_http_handler_1 = require("@smithy/node-http-handler");
class DynamodbFetcher {
    constructor(tableName, getDdbKey, batchOptions) {
        var _a;
        this.batch = 0;
        this.tableName = tableName;
        this.getDdbKey = getDdbKey;
        this.batchOptions = batchOptions;
        this.ddb = lib_dynamodb_1.DynamoDBDocument.from(new client_dynamodb_1.DynamoDBClient({
            region: (_a = process.env.AWS_REGION) !== null && _a !== void 0 ? _a : "us-east-1",
            maxAttempts: 3,
            requestHandler: new node_http_handler_1.NodeHttpHandler({
                connectionTimeout: 2000,
                requestTimeout: 5000,
                httpsAgent: new https_1.default.Agent({
                    ciphers: 'ALL',
                }),
            }),
        }), {
            marshallOptions: {
                convertEmptyValues: true,
                removeUndefinedValues: true
            }
        });
    }
    keyToString(key) {
        return Object.keys(key).sort().map(k => {
            return `${k}:${key[k]}`;
        }).join("/");
    }
    async join(events) {
        let myBatch = this.batch++;
        let params = {
            RequestItems: {
                [this.tableName]: {
                    Keys: []
                }
            },
            "ReturnConsumedCapacity": 'TOTAL'
        };
        let keys = {};
        let work = [];
        // TODO: Break up requests into 100
        events.map(e => {
            let key = this.getDdbKey(e);
            if (key != null) {
                let k = this.keyToString(key);
                e.ddbKey = k;
                if (!(k in keys)) {
                    keys[k] = true;
                    params.RequestItems[this.tableName].Keys.push(key);
                    if (params.RequestItems[this.tableName].Keys.length === 100) {
                        work.push(params);
                        params = {
                            RequestItems: {
                                [this.tableName]: {
                                    Keys: []
                                }
                            },
                            "ReturnConsumedCapacity": 'TOTAL'
                        };
                    }
                }
            }
        });
        if (params.RequestItems[this.tableName].Keys.length > 0) {
            work.push(params);
        }
        let lookup = {};
        if (work.length > 0) {
            let keyFields = Object.keys(work[0].RequestItems[this.tableName].Keys[0]);
            console.log("query", myBatch, work.length);
            let c = 0;
            // eslint-disable-next-line @typescript-eslint/no-this-alias
            let self = this;
            await (0, util_1.promisify)(async_1.default.eachLimit).call(async_1.default, work, 10, async function (params) {
                var _a, _b;
                let myC = c++;
                let extData = await self.ddb.batchGet(params);
                (_b = (_a = extData.Responses) === null || _a === void 0 ? void 0 : _a[self.tableName]) === null || _b === void 0 ? void 0 : _b.forEach(entry => {
                    let keyObject = keyFields.reduce((a, field) => { a[field] = entry[field]; return a; }, {});
                    let key = self.keyToString(keyObject);
                    lookup[key] = entry;
                });
                console.log("query", myC, "done");
            });
            console.log("joined", myBatch, Object.values(lookup).length);
        }
        else {
            console.log("joined", myBatch, "no work");
        }
        return events.map(e => {
            e.joinedData = lookup[e.ddbKey];
            //delete e.ddbKey;
            return e;
        });
    }
}
exports.DynamodbFetcher = DynamodbFetcher;
//# sourceMappingURL=DynamodbFetcher.js.map