export interface ConfigOptions {
    stageEnvVar?: string;
    stage?: string;
    regionEnvVar?: string;
    region?: string;
    optional?: boolean;
}
export type ResourceReferenceType = "string" | "number" | "float" | "integer" | "int" | "dynamic" | string;
declare abstract class Resource implements ResourceReference {
    key: string;
    type: ResourceReferenceType;
    options?: ConfigOptions;
    service: string;
    constructor(key: string, type?: ResourceReferenceType, options?: ConfigOptions);
}
export declare class CfResource extends Resource {
    service: string;
}
export declare class StackResource extends Resource {
    service: string;
}
export declare class SecretResource extends Resource {
    service: string;
}
export interface ResourceReference {
    service: string;
    key: string;
    type: ResourceReferenceType;
    options?: ConfigOptions;
}
export declare type ConfigurationData = ResourceReference | number | string | {
    [key: string]: ConfigurationData;
};
export declare class ConfigurationBuilder<T> {
    private data?;
    constructor(data?: ConfigurationData);
    build(options?: ConfigOptions): T;
    private resolve;
    private resolveReference;
    private isResourceReference;
    static Resolvers: Record<string, (ref: ResourceReference, cache: any) => any>;
}
export declare function resolveKeywords(template: string, data: any): string;
export declare function getDataSafe(data?: {}, path?: string): any;
export declare function inferTypes(node: any): any;
export {};
