"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.determineReadHooks = exports.addReadHooks = exports.createFastS3ReadHooks = exports.getAllFiles = void 0;
const path_1 = __importStar(require("path"));
const worker_pool_1 = require("./worker-pool");
const os_1 = require("os");
const async = __importStar(require("async"));
const fs_1 = require("fs");
const stream_1 = require("stream");
const streamUtil = __importStar(require("../../streams"));
const zlib_1 = require("zlib");
//import { Worker } from 'worker_threads';
let logger = require("leo-logger")("leo-stream-helper");
const MB = 1024 * 1024;
const GB = MB * 1024;
let { parseTaskModuleContent, downloadTaskModuleContent } = JSON.parse((0, zlib_1.gunzipSync)(Buffer.from(require("./worker-thread-content.js"), "base64")).toString());
/**
 * Verify that a task module exists.  If it doesn't save the internal content in a location
 * so that it does exists.  Otherwise worker threads won't work
 * @param taskPath path to that task. can be null
 * @param tmpDir directory to store code if needed
 * @param taskModule code of the task to save if needed
 * @param filename filename to use to look for the module
 * @returns path to the task
 */
function verifyTaskModule(taskPath, tmpDir, taskModule, filename) {
    if (taskPath == null) {
        // Check if the entry is available (webpack may package it away)
        let chain = [
            path_1.default.resolve(__dirname, `./${filename}`),
            path_1.default.resolve("./", `./${filename}`)
        ];
        for (let f of chain) {
            let exists = (0, fs_1.existsSync)(f);
            //console.log("Checking File:", f, exists);
            if (exists) {
                taskPath = f;
                break;
            }
        }
        // If there still isn't a module copy the source to the tmp directory
        if (!(0, fs_1.existsSync)(taskPath)) {
            // Copy the source to the tmp code directory
            taskPath = path_1.default.resolve(tmpDir, `code/${filename}`);
            //if (!existsSync(taskPath) || (Date.now() - statSync(taskPath).mtime.valueOf()) > (1000 * 60 * 5)) {
            (0, fs_1.mkdirSync)((0, path_1.dirname)(taskPath), { recursive: true });
            let fileContent = [
                taskModule.toString(),
                `typeof taskModule === "function" && taskModule(require)`
            ].join("\n");
            //logger.log(filename, "\n", fileContent);
            (0, fs_1.writeFileSync)(taskPath, fileContent);
            //}
        }
    }
    logger.debug("Task Path", filename, taskPath);
    return taskPath;
}
function getAllFiles(dirPath, arrayOfFiles) {
    let files = (0, fs_1.existsSync)(dirPath) ? (0, fs_1.readdirSync)(dirPath) : [];
    arrayOfFiles = arrayOfFiles || [];
    files.forEach(function (file) {
        let fullPath = path_1.default.resolve(dirPath, file);
        let stat = (0, fs_1.statSync)(fullPath);
        if (stat.isDirectory()) {
            arrayOfFiles = getAllFiles(fullPath, arrayOfFiles);
        }
        else {
            stat.fullpath = fullPath;
            arrayOfFiles.push(stat);
        }
    });
    return arrayOfFiles;
}
exports.getAllFiles = getAllFiles;
const memSizes = ["Bytes", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"];
function convertBytes(bytes) {
    let sign = Math.sign(bytes);
    bytes = Math.abs(bytes);
    if (bytes == 0) {
        return "n/a";
    }
    const i = (Math.floor(Math.log(bytes) / Math.log(1024)));
    if (i == 0) {
        return (bytes * sign) + " " + memSizes[i];
    }
    return ((bytes * sign) / Math.pow(1024, i)).toFixed(1) + " " + memSizes[i];
}
/**
 * Creates read hooks that use worker threads for downloading and parsing
 * @param settings
 * @returns
 */
function createFastS3ReadHooks(settings) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    settings = {
        downloadThreads: Math.max(1, (0, os_1.cpus)().length - 1),
        payloadAtEnd: false,
        unzipFiles: false,
        saveFiles: false,
        tmpDir: process.env.RSTREAMS_TMP_DIR || "/tmp/rstreams",
        ...settings
    };
    let batchId = 0;
    let queryId = 0;
    let taskId = 0;
    let downloadThreads = (_a = settings.downloadThreads) !== null && _a !== void 0 ? _a : Math.max(1, (0, os_1.cpus)().length - 1);
    let parseThreads = (_b = settings.parseThreads) !== null && _b !== void 0 ? _b : Math.max(1, (0, os_1.cpus)().length - 1);
    let downloadTaskPath = verifyTaskModule(settings.downloadTaskPath, settings.tmpDir, downloadTaskModuleContent, "download-task-module.js");
    let parseTaskPath = verifyTaskModule(settings.parseTaskPath, settings.tmpDir, parseTaskModuleContent, "parse-task-module.js");
    let pool = new worker_pool_1.WorkerPool("Download", downloadTaskPath, downloadThreads, {
        payloadAtEnd: settings.payloadAtEnd,
        unzipFiles: settings.unzipFiles,
        awsS3Config: {
            ...settings.awsS3Config,
            // copy over only the credentials options, other credential fields won't serialize
            ...(((_c = settings.awsS3Config) === null || _c === void 0 ? void 0 : _c.credentials) ? {
                credentials: {
                    accessKeyId: ((_d = settings.awsS3Config) === null || _d === void 0 ? void 0 : _d.credentials).accessKeyId,
                    secretAccessKey: ((_e = settings.awsS3Config) === null || _e === void 0 ? void 0 : _e.credentials).secretAccessKey,
                    sessionToken: ((_f = settings.awsS3Config) === null || _f === void 0 ? void 0 : _f.credentials).sessionToken,
                }
            } : undefined)
        },
        concurrency: 10,
    });
    let workerLink = {};
    let parsePool = new worker_pool_1.WorkerPool("Parse", parseTaskPath, settings.parallelParse ? parseThreads : 0, {
        parser: (_g = settings.parseTaskParser) === null || _g === void 0 ? void 0 : _g.parser,
        parserOpts: (_h = settings.parseTaskParser) === null || _h === void 0 ? void 0 : _h.opts,
        bufferSize: (_j = settings.parseTaskParser) === null || _j === void 0 ? void 0 : _j.bufferSize, // ?? settings.parallelParseBufferSize
    }, {
        data: async (worker, result) => {
            let done = () => {
                worker.postMessage({
                    event: "pushed",
                    data: {
                        id: result.id
                    }
                });
            };
            let stream = workerLink[result.id];
            let data = Array.isArray(result.data) ? result.data : [result.data];
            for (let i = 0; i < data.length; i++) {
                if (!stream.write(data[i])) {
                    logger.debug(result.id, "parsePool Backpressure - start");
                    await new Promise(resolve => stream.once("drain", () => {
                        logger.debug(result.id, "parsePool Backpressure - done");
                        resolve(undefined);
                    }));
                }
            }
            done();
        }
    });
    function poolStream(pool, id, source, queue) {
        let pass = streamUtil.passthrough({
            objectMode: true,
            highWaterMark: 10000
        });
        workerLink[id] = pass;
        pool.runTask({
            event: "parse",
            data: {
                id,
                //filePath: file,
                ...source,
                queue
            }
        }, (err, result) => {
            err = err || result.error;
            delete workerLink[id];
            if (err) {
                logger.error(err);
                pass.emit("error", err);
            }
            else {
                pass.end();
            }
        });
        return pass;
    }
    let availableDiskSpace = (_k = settings.availableDiskSpace) !== null && _k !== void 0 ? _k : Math.floor(MB * (500 * 0.8));
    let s3Dir = path_1.default.resolve(settings.tmpDir, `s3/`);
    let cachedS3Files = getAllFiles(s3Dir);
    let usedDiskSpace = 0;
    function freeDiskSpace(size) {
        let prev = usedDiskSpace;
        usedDiskSpace -= size;
        if (usedDiskSpace < availableDiskSpace && prev >= availableDiskSpace) {
            logger.debug("Download Queue - Resuming");
            downloadQueue.resume();
        }
    }
    function useDiskSpace(size) {
        let prev = usedDiskSpace;
        usedDiskSpace += size;
        if (usedDiskSpace >= availableDiskSpace && prev < availableDiskSpace) {
            logger.debug("Download Queue - Pausing");
            downloadQueue.pause();
        }
    }
    let downloadQueue = async.queue(function (task, callback) {
        logger.debug(task.id, "download task start");
        if (downloadQueue.isExiting) {
            logger.debug(task.id, "download task end - exiting");
            callback();
        }
        else if ((0, fs_1.existsSync)(task.filePath)) {
            logger.debug(task.id, "download task end - exists");
            callback();
        }
        else {
            let filesToDownload = Array.isArray(task.s3) ? task.s3.length : 1;
            let diskSpaceNeededToDownload = filesToDownload == 1 ? task.fileSize : task.fileSize * 2;
            logger.debug(task.id, `Disk Space Required file: ${convertBytes(task.fileSize)} needed: ${convertBytes(diskSpaceNeededToDownload)}, files: ${filesToDownload}, used: ${convertBytes(usedDiskSpace)}`, task.filePath);
            useDiskSpace(diskSpaceNeededToDownload);
            let error;
            pool.runTaskAsync(task)
                .then((data) => {
                error = data.error;
            })
                .catch((err) => {
                error = err;
            })
                .finally(() => {
                logger.debug(task.id, "download task end");
                if (filesToDownload > 1) {
                    // clean up the tmp files space used to download (they have been deleted already)
                    logger.debug("Disk Space Cleaning Extra", task.id, task.fileSize, task.filePath);
                    freeDiskSpace(task.fileSize);
                }
                if (error) {
                    // on error we remove the file so cleanup the size used
                    logger.debug("Disk Space Cleaning Error", task.id, task.fileSize, task.filePath, Object.assign(new Error(), error));
                    freeDiskSpace(task.fileSize);
                    if ((0, fs_1.existsSync)(task.filePath)) {
                        (0, fs_1.unlinkSync)(task.filePath);
                    }
                }
                //logger.log(task.id, "done", err || "", task.filePath);
                callback(downloadQueue.isExiting ? undefined : error);
            });
        }
    }, downloadThreads);
    downloadQueue.fullDrain = async function () {
        let tasks = downloadQueue.workersList();
        logger.debug("download queue drain start", tasks.length, !this.idle());
        // tasks.forEach(taskWrapper => {
        // 	let task = taskWrapper.data;
        // 	logger.log("download task destroy:", task.id, JSON.stringify(task));
        // 	// if (task.downloadStream) {
        // 	// 	task.downloadStream.on("error", (err) => {
        // 	// 		// Ignore errors.  This stream was never used
        // 	// 		logger.debug("close_remaining_s3_streams.error Error closing s3 stream:", task.s3.key, err);
        // 	// 		logger.log("destroy download error:", task.id);
        // 	// 	});
        // 	// 	logger.debug("close_remaining_s3_streams.destroy closing s3 stream:", task.s3.key);
        // 	// 	logger.log("destroy download:", task.id);
        // 	// 	task.downloadStream.destroy();
        // 	// }
        // });
        // logger.log("download queue draining", tasks.length, !this.idle());
        //logger.log("HERE", downloadQueue.length());
        this.kill();
        if (!this.idle()) {
            await new Promise(resolve => downloadQueue.drain = () => resolve(undefined));
        }
        logger.debug("download queue drain end");
    };
    let deleteQueue = async.queue(function (task, callback) {
        logger.debug(task, "delete task start");
        (0, fs_1.unlink)(task, (err) => {
            logger.debug(task, "delete task done", err || "");
            // Don't fail when delete fails
            callback();
        });
    }, downloadThreads);
    deleteQueue.fullDrain = async function () {
        let tasks = deleteQueue.workersList();
        logger.debug("delete queue drain start", tasks.length);
        //this._tasks.empty()
        if (!this.idle()) {
            await new Promise(resolve => deleteQueue.drain = () => resolve(undefined));
        }
        logger.debug("delete queue drain end");
    };
    useDiskSpace(cachedS3Files.reduce((t, f) => t + f.size, 0));
    let startingDiskSpace = usedDiskSpace;
    let endingDiskSpace;
    logger.debug(`Total Disk: ${convertBytes(availableDiskSpace)}, Used: ${convertBytes(usedDiskSpace)}, Free: ${convertBytes(availableDiskSpace - usedDiskSpace)}`);
    let parseStreamId = 0;
    let totalGzipBytesSeen = 0;
    return {
        getExtraMetaData() {
            return {
                availableDiskSpace: availableDiskSpace,
                startingDiskSpace: startingDiskSpace,
                enddingDiskSpace: endingDiskSpace,
                ...settings
            };
        },
        onStart(data) {
            logger.debug("Hook onStart", data.queue, data.eid);
            if (usedDiskSpace >= availableDiskSpace) {
                logger.time("Disk Space Cleanup - onStart");
                let minNeededFile = data.eid.replace(/\//g, "_");
                let pergeTargetDiskSpace = Math.floor(availableDiskSpace * 0.8);
                logger.debug("Disk Space Cleanup - Removing file before start", data.eid, convertBytes(pergeTargetDiskSpace));
                for (let i = 0; i < cachedS3Files.length; i++) {
                    let f = cachedS3Files[i];
                    let name = (0, path_1.basename)(f.fullpath);
                    if (name < minNeededFile) {
                        logger.debug("Disk Space Cleanup - Removing file before current position", data.eid, name);
                        (0, fs_1.unlinkSync)(f.fullpath);
                        freeDiskSpace(f.size);
                    }
                    else {
                        break;
                    }
                }
                for (let i = cachedS3Files.length - 1; i >= 0 && usedDiskSpace >= pergeTargetDiskSpace; i--) {
                    let f = cachedS3Files[i];
                    let name = (0, path_1.basename)(f.fullpath);
                    logger.debug("Disk Space Cleanup - Removing file far ahead of current position", data.eid, name);
                    (0, fs_1.unlinkSync)(f.fullpath);
                    freeDiskSpace(f.size);
                }
                logger.timeEnd("Disk Space Cleanup - onStart");
            }
        },
        async onEnd() {
            logger.debug("Hook onEnd");
            downloadQueue.isExiting = true;
            await Promise.allSettled([
                downloadQueue.fullDrain(),
                deleteQueue.fullDrain()
            ]);
            // Close pool after everything drains
            await pool.close();
            await parsePool.close();
            logger.debug("Hook onEnd - complete");
            logger.debug(`Total Disk: ${convertBytes(availableDiskSpace)}, Used: ${convertBytes(usedDiskSpace)}, Free: ${convertBytes(availableDiskSpace - usedDiskSpace)}`);
            endingDiskSpace = usedDiskSpace;
        },
        // createSplitParseStream2(JSONparse) {
        // 	return split((value) => {
        // 		try {
        // 			let obj = JSONparse(value);
        // 			if (obj != null && obj.size == null) {
        // 				obj.size = Buffer.byteLength(value);
        // 			}
        // 			return obj;
        // 		} catch (e) {
        // 			//If we cancel the download early, we don't want to die here.
        // 			return null;
        // 		}
        // 	});
        // },
        createSplitParseStream(JSONparse, record) {
            if ((record.s3 || record.s3Like) && settings.parallelParse) {
                return new stream_1.PassThrough({ objectMode: true });
            }
            else if (record.gzip && settings.parallelParse) {
                // TODO: can I create a split and parse for gzip?
                return null;
            }
            else {
                return null;
            }
        },
        createS3Stream(streamRecord, index) {
            let _parts = streamRecord.start.split(/-/);
            let idOffset = parseInt(_parts[1]);
            if (streamRecord.s3Like) {
                let pass = poolStream(parsePool, ++parseStreamId, { gzip: streamRecord.gzip }, streamRecord.event);
                pass.isUnzipped = true;
                pass.idOffset = idOffset;
                return {
                    get: function () {
                        return pass;
                    },
                    on: function (...args) {
                        return pass.on(...args);
                    },
                    destroy: function (...args) {
                        return pass.destroy(...args);
                    }
                };
            }
            let localFile = streamRecord.localFile.filePath;
            let parallelParse = settings.parallelParse;
            let pass = new stream_1.PassThrough({ objectMode: parallelParse });
            pass.idOffset = idOffset;
            pass.isUnzipped = settings.unzipFiles || parallelParse;
            let fileStream;
            let isDestroyed = false;
            let hasPiped = false;
            let passpipe = pass.pipe.bind(pass);
            pass.pipe = function (dest, opts) {
                hasPiped = true;
                return passpipe(dest, opts);
            };
            streamRecord.localFile.readyPromise.then(() => {
                if (!isDestroyed) {
                    fileStream = parallelParse ? poolStream(parsePool, ++parseStreamId, { filePath: localFile }, streamRecord.event) : (0, fs_1.createReadStream)(localFile);
                    streamUtil.pipe(fileStream, pass, () => {
                        let files = streamRecord.localFile.unlinkOnStreamClose || [];
                        if (files.length > 0) {
                            logger.debug("Disk Space Cleaning unlinkOnStreamClose", streamRecord.gzipSize, localFile);
                            freeDiskSpace(streamRecord.gzipSize);
                            (files).forEach(f => {
                                deleteQueue.push(f);
                                //existsSync(f) && unlinkSync(f);
                            });
                        }
                    });
                }
            }).catch(err => {
                if (!isDestroyed) {
                    if (hasPiped) {
                        pass.emit("error", err);
                    }
                    else {
                        let pipe = pass.pipe.bind(pass);
                        pass.pipe = function (dest, opts) {
                            let ret = pipe(dest, opts);
                            dest.emit("error", err);
                            return ret;
                        };
                    }
                }
            });
            return {
                get: function () {
                    return pass;
                },
                on: function (...args) {
                    return pass.on(...args);
                },
                destroy: function (...args) {
                    if (fileStream) {
                        return fileStream.destroy(...args);
                    }
                    else {
                        return pass.destroy(...args);
                    }
                }
            };
        },
        onGetEvents(streamRecords) {
            logger.debug("onGetEvents", streamRecords.length);
            let mergeVersion = settings.mergeFileVersion;
            if (mergeVersion == 1 && settings.mergeFileSize != 0) {
                streamRecords = this.onGetEventsMerge(streamRecords);
            }
            else {
                this.onGetEventsAddDownload(streamRecords);
            }
            let id = queryId++;
            streamRecords.queryId = id;
            logger.debug(id, "Start Query");
            return streamRecords;
        },
        onGetEventsMerge(streamRecords) {
            var _a;
            logger.debug("onGetEventsMerge", streamRecords.length);
            let ret = [];
            let maxFileSize = (_a = settings.mergeFileSize) !== null && _a !== void 0 ? _a : (5 * 1024 * 1024);
            streamRecords.forEach(r => {
                totalGzipBytesSeen += r.gzipSize;
                if (r.s3 && !Array.isArray(r.s3)) {
                    let last = ret[ret.length - 1];
                    r.s3.start = r.start;
                    r.s3.end = r.end;
                    r.s3.gzipSize = r.gzipSize;
                    if (totalGzipBytesSeen < (1 * 1024 * 1024) || last == null || !last.s3 || (last.gzipSize + r.gzipSize) > maxFileSize || last.s3.length > 10) {
                        r.s3 = [r.s3];
                        ret.push(r);
                    }
                    else {
                        last.s3.push(r.s3);
                        last.offsets = last.offsets.concat(r.offsets);
                        last.end = r.end;
                        last.records += r.records;
                        last.gzipSize += r.gzipSize;
                        last.size += r.size;
                    }
                }
                else {
                    ret.push(r);
                }
            });
            this.onGetEventsAddDownload(ret);
            return ret;
        },
        onGetEventsAddDownload(streamRecords) {
            logger.debug("onGetEventsAddDownload", streamRecords.length);
            streamRecords.forEach(r => {
                if (r.s3) {
                    let filePath = path_1.default.resolve(settings.tmpDir, `s3/${r.event}/${r.filePrefix || ""}${((r.end)).replace(/\//g, "_") + "-0-" + r.gzipSize}.jsonl.gz`);
                    let readyPromise = createPromiseResolver();
                    let id = taskId++;
                    logger.debug(id, "task push");
                    r.localFile = {
                        id,
                        filePath,
                        readyPromise
                    };
                    readyPromise.catch(() => {
                        // Do nothing.  The error is later emitted if piped
                    });
                    let task = {
                        id: id,
                        s3: r.s3,
                        //inMem: false,
                        filePath,
                        fileSize: r.gzipSize
                        //data: undefined
                    };
                    //r.stream = sdk.streams.passThrough();
                    downloadQueue.push(task, (err) => {
                        logger.debug(id, "download task queue end", err || "");
                        if (err) {
                            readyPromise.reject(err);
                        }
                        else {
                            r.localFile.data = task.data;
                            readyPromise.resolve();
                        }
                    });
                }
                else if (r.gzip && settings.parallelParse && settings.parallelParseGzip) {
                    // TODO: Should we pass gzip like s3 to the parallel parser
                    r.s3Like = true;
                }
            });
            // let id = queryId++;
            // (streamRecords as any).queryId = id;
            // logger.log(id, "Start Query");
            // return streamRecords;
        },
        onBatchStart(streamRecords) {
            let returnRecords = streamRecords;
            let id = batchId++;
            let rr = returnRecords;
            rr.batchId = id;
            logger.debug(id, "Start batch", rr.queryId);
        },
        onRecordEnd(streamRecord) {
            //Delete local files
            if (!settings.saveFiles && streamRecord.localFile != null) {
                logger.debug("Disk Space Cleaning onRecordEnd", streamRecord.gzipSize, streamRecord.localFile.filePath);
                freeDiskSpace(streamRecord.gzipSize);
                deleteQueue.push(streamRecord.localFile.filePath);
            }
        }
    };
}
exports.createFastS3ReadHooks = createFastS3ReadHooks;
/**
 * Lambda with 10 GB
 * dthreads: 4 | 2
 * pthreads: even 4 | 8 | 2
 * parseBufferSize: 2MB | 1MB (large objects perfer smaller values)
 * fast_s3_parallel_fatch_max_bytes: 100MB (slower parsers eg. JSON.parse prefer smaller values 50MB)
 * mergeFileSize: No Merge //even 2MB | 5MB (2MB by a little)
**/
let totalCpus = (0, os_1.cpus)().length;
let workerCpus = totalCpus - 1;
let totalMemory = (parseInt(process.env.AWS_LAMBDA_FUNCTION_MEMORY_SIZE, 10) * MB) || (0, os_1.totalmem)();
let lambdaDefaultsByMemory = [
    {
        memory: 10 * GB,
        downloadThreads: 2,
        parseThreads: 4,
        parseBufferSize: 2 * MB,
        parallelFetchMax: 50 * MB
    }, {
        memory: 0,
        // Split CPUs between download and parse
        downloadThreads: Math.max(1, Math.floor(Math.max(1, workerCpus * 0.3))),
        parseThreads: Math.max(1, workerCpus - Math.floor(Math.max(1, workerCpus * 0.3))),
        parseBufferSize: 2 * MB,
        parallelFetchMax: 5 * MB
    }
].sort((a, b) => b.memory - a.memory);
function getLambdaDefaults(memoryAvailable) {
    return lambdaDefaultsByMemory.find(d => d.memory <= memoryAvailable) || lambdaDefaultsByMemory[lambdaDefaultsByMemory.length - 1];
}
/**
 * Addes configured read hooks to the read options
 * @param readOpts
 * @param partialHookSettings
 * @param extraConfig
 * @returns
 */
function addReadHooks(readOpts, partialHookSettings, extraConfig) {
    Object.assign(readOpts, determineReadHooks(readOpts, partialHookSettings, extraConfig));
    return readOpts;
}
exports.addReadHooks = addReadHooks;
/**
 *
 * @param settings
 * @param partialHookSettings
 * @param extraConfig
 * @returns
 */
function determineReadHooks(settings, partialHookSettings, extraConfig) {
    var _a, _b;
    let extraConfiguration = {
        tmpDir: process.env.RSTREAMS_TMP_DIR || "/tmp/rstreams"
    };
    let maybeSdk = extraConfig;
    if (maybeSdk.streams != null && maybeSdk.configuration != null) {
        extraConfiguration.tmpDir = maybeSdk.streams.tmpDir;
        extraConfiguration.awsS3Config = {
            region: maybeSdk.configuration.aws.region,
            credentials: maybeSdk.configuration.credentials
        };
    }
    else {
        extraConfiguration = {
            ...extraConfiguration,
            ...extraConfig
        };
    }
    let defaultsFromMem = getLambdaDefaults(totalMemory);
    let parseTaskParser = typeof settings.parser === "string" ? {
        parser: settings.parser,
        bufferSize: MB,
        ...partialHookSettings === null || partialHookSettings === void 0 ? void 0 : partialHookSettings.parseTaskParser,
        opts: {
            parser: ((_a = partialHookSettings === null || partialHookSettings === void 0 ? void 0 : partialHookSettings.parseTaskParser) === null || _a === void 0 ? void 0 : _a.parser) || settings.parser,
            ...settings.parserOpts,
            ...(_b = partialHookSettings === null || partialHookSettings === void 0 ? void 0 : partialHookSettings.parseTaskParser) === null || _b === void 0 ? void 0 : _b.opts
        },
    } : undefined;
    let parallelParse = parseTaskParser != null;
    let downloadThreads = defaultsFromMem.downloadThreads;
    let parseThreads = defaultsFromMem.parseThreads;
    let saveFiles = false;
    let tmpDir = extraConfiguration.tmpDir;
    // Not using them
    let mergeFileSize;
    let mergeFileVersion;
    let awsS3Config = extraConfiguration.awsS3Config;
    let hookSettings = {
        parallelParse,
        //parallelParseBufferSize,
        downloadThreads,
        parseThreads,
        saveFiles,
        tmpDir,
        mergeFileSize,
        mergeFileVersion,
        awsS3Config,
        ...(partialHookSettings || {}),
        // last because it is a sub object that we don't want overriden 
        parseTaskParser,
    };
    let readOpts = {
        hooks: createFastS3ReadHooks(hookSettings),
        _hookSettings: hookSettings
    };
    if (defaultsFromMem.parallelFetchMax > 0) {
        readOpts.stream_query_limit = 1000;
        readOpts.fast_s3_read = true;
        readOpts.fast_s3_read_parallel_fetch_max_bytes = readOpts.fast_s3_read_parallel_fetch_max_bytes || defaultsFromMem.parallelFetchMax;
    }
    // console.log("Hook Params New:", JSON.stringify({
    // 	...readOpts,
    // 	hooks: hookSettings
    // }, null, 2));
    return readOpts;
}
exports.determineReadHooks = determineReadHooks;
function createPromiseResolver() {
    let resolve;
    let reject;
    let promise = new Promise((res, rej) => {
        reject = rej;
        resolve = res;
    });
    promise.resolve = resolve;
    promise.reject = reject;
    return promise;
}
//# sourceMappingURL=leo-stream-helper.js.map