"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createParser = exports.parsers = exports.fieldParsers = exports.ParserName = exports.FastJsonPlus = exports.FastParseType = void 0;
/* eslint-disable @typescript-eslint/ban-ts-comment */
const fast_json_1 = require("fast-json");
var FastParseType;
(function (FastParseType) {
    FastParseType["Number"] = "number";
    FastParseType["String"] = "string";
    FastParseType["Eid"] = "eid";
    FastParseType["Raw"] = "raw";
    FastParseType["Json"] = "json";
    FastParseType["Array"] = "array";
    FastParseType["Object"] = "object";
})(FastParseType || (exports.FastParseType = FastParseType = {}));
// Overriding a private function to add isString feature.
class FastJsonPlus extends fast_json_1.FastJson {
    constructor(opts) {
        super(opts);
        this.isLastPrimitiveAString = false;
        // Override the private function with our own local wrapper
        this["emitPrimitiveOrString"] = this._emitPrimitiveOrString;
    }
    parse(input) {
        // Create an object for parsing
        let obj = {
            __unparsed_value__: input
        };
        this.currentObj = obj;
        // Write the current input
        this.write(input);
        // remove the current object and return
        delete this.currentObj;
        return obj;
    }
    getPath() {
        // Some fields are private so we have to fake access to them
        const fastJson = this;
        return fastJson.events.node.getPath().map((f, i) => {
            // Get the real field name for wildcards
            if (f == "*") {
                if (fastJson.stack.length > i + 1) {
                    // If it is in the stack we can use the stack key
                    f = fastJson.stack[i + 1].key;
                }
                else {
                    // If it isn't in the stack we use the latest key
                    f = fastJson.getKey(fastJson.currentObj.__unparsed_value__);
                }
            }
            return f;
        });
    }
    // Override private function to determine if the last permitive is a string or not
    _emitPrimitiveOrString(data, start, end) {
        this.isLastPrimitiveAString = data[start - 1] == '"' && data[end] == '"';
        return super["emitPrimitiveOrString"](data, start, end);
    }
}
exports.FastJsonPlus = FastJsonPlus;
var ParserName;
(function (ParserName) {
    ParserName["JsonParse"] = "JSON.parse";
    ParserName["Empty"] = "empty";
    ParserName["FastJson"] = "fast-json";
})(ParserName || (exports.ParserName = ParserName = {}));
exports.fieldParsers = {
    [FastParseType.String]: { parse: (value) => value },
    [FastParseType.Number]: { parse: (value) => Number(value) },
    [FastParseType.Eid]: { parse: (value) => value.toString().startsWith("z/") ? value : parseInt(value.toString(), 10) },
    [FastParseType.Raw]: { set: (field, value, setFn) => setFn(field, value, "_raw") },
    // These are handled the same way.  Just JSON strings to parse
    [FastParseType.Json]: { parse: (value) => JSON.parse(value.toString()) },
    [FastParseType.Array]: { parse: (value) => JSON.parse(value.toString()) },
    [FastParseType.Object]: { parse: (value) => JSON.parse(value.toString()) },
};
// Factory of parsers so that they can be created with any config needed
exports.parsers = {
    [ParserName.JsonParse]: () => JSON.parse,
    [ParserName.Empty]: () => (input) => {
        return {
            id: "unknown",
            payload: {}
        };
    },
    [ParserName.FastJson]: (settings) => {
        var _a;
        let fastJson = new FastJsonPlus({ pathSeparator: settings.pathSeparator });
        let allFields = 0;
        let visitedFields = 0;
        let wrapperFieldsDefs = [{
                field: "id"
            }, {
                field: "event"
            },
            // {
            // 	field: "eid",
            // 	type: FastParseType.Eid
            // },
            {
                field: "event_source_timestamp",
                type: FastParseType.Number
            },
            // {
            // 	field: "timestamp",
            // 	type: FastParseType.Number
            // },
            // {
            // 	field: "correlation_id",
            // 	type: FastParseType.Raw
            // },
            // {
            // 	field: "payload",
            // 	type: FastParseType.Raw
            // }
        ];
        /**
         * Example from payload
         * 	{
         * 		field: "payload.new.supplier_id",
         * 		type: FastParseType.Number
         * 	}
         */
        let allFieldsDefs = wrapperFieldsDefs.concat((_a = settings.fields) !== null && _a !== void 0 ? _a : []);
        const ARRAY_TYPE = 1;
        function set(field, value, suffix = "") {
            let keys = fastJson.getPath();
            let last = keys.pop();
            // Detect if this is an array and use [] instead of {}
            let c = keys.reduce((c, f, i) => c[f] = c[f] || (fastJson.stack[i + 1].type == ARRAY_TYPE ? [] : {}), fastJson.currentObj);
            if (Array.isArray(c)) {
                c[parseInt(last, 10)] = value;
            }
            else {
                c[last + suffix] = value;
            }
        }
        let allowSkip = true;
        function trySetSkip() {
            if (allowSkip && visitedFields == allFields) {
                fastJson.skip();
            }
        }
        let defaultFieldSet = ((a, b) => set(a, b));
        allFieldsDefs.forEach(def => {
            allFields++;
            let fieldProcessor = exports.fieldParsers[def.type] || exports.fieldParsers.string;
            let fieldParser = fieldProcessor.parse || exports.fieldParsers.string.parse;
            let fieldSet = fieldProcessor.set || defaultFieldSet;
            // If wildcards are used we can't determine if all fields were visited
            // so disable skipping
            if (def.field.includes("*")) {
                allowSkip = false;
            }
            fastJson.on(def.field, (value) => {
                // If the value is string null then use JS null
                if (value == "null" && !fastJson.isLastPrimitiveAString) {
                    fieldSet(def.field, null, set);
                }
                else {
                    fieldSet(def.field, fieldParser(value), set);
                }
                visitedFields++;
                trySetSkip();
            });
        });
        //console.log(allFieldsDefs.map(a => a.field));
        return (input) => {
            visitedFields = 0;
            return fastJson.parse(input);
        };
    }
};
const requireFn = typeof __webpack_require__ === "function" ? __non_webpack_require__ : require;
// Create a parser and handle eid commands
// Eid Commands are needed when merging multiple records
// LeoStream records together as they have different starting eids
function createParser(params) {
    let JSONparseInner;
    if (typeof params.parser === "string") {
        let parseFn = exports.parsers[params.parser];
        if (parseFn == null) {
            //console.log("PARSER:", params.parser);
            // If it isn't one of the default parsers
            // it should be an external module that we can require
            // with a function to call to get the parser
            parseFn = requireFn(params.parser);
            if (typeof parseFn !== "function") {
                let lib = parseFn;
                parseFn = lib.default;
                if (typeof parseFn !== "function") {
                    parseFn = parseFn.parser;
                }
            }
            parseFn = parseFn || exports.parsers[ParserName.JsonParse];
        }
        //console.log("PARSER context:", parseFn.toString());
        // Create the inner parser and pass in the parser options
        if (parseFn.length == 1) {
            JSONparseInner = (parseFn)(params.opts);
        }
        else {
            JSONparseInner = (parseFn)(params.opts.botId, params.opts.queue, params.opts);
        }
    }
    else {
        JSONparseInner = params.parser;
    }
    JSONparseInner = JSONparseInner || JSON.parse;
    return (str) => {
        if (str.startsWith("__cmd:eid__")) {
            let cmd = JSON.parse(str.replace("__cmd:eid__", ""));
            cmd._cmd = "setBaseEid";
            return cmd;
        }
        let r = JSONparseInner(str);
        // Add size in bytes if it doesn't exist
        if (r != null && r.size == null) {
            r.size = Buffer.byteLength(str);
        }
        return r;
    };
}
exports.createParser = createParser;
//# sourceMappingURL=parser-util.js.map